#include "connection.h"
#include "error.h"
#include <arpa/inet.h>
#include <netdb.h>

namespace Librescue {
  Connection::Connection() {
  }

  Connection::~Connection() {
  }

  /*
  void Connection::send(std::string string) {
	Output out;
	out.writeString(string);
	send(out);
  }

  std::string Connection::receive() {
	Input in;
	receive(in);
	return in.readString();
  }
  */

  bool Connection::isDataAvailable(std::vector<Connection*> connections) {
	for (std::vector<Connection*>::iterator it = connections.begin();it!=connections.end();++it) {
	  Connection* c = *it;
	  if (c->isDataAvailable(1)) return true;
	}
	return false;
  }

  bool Connection::isDataAvailable(std::vector<Connection*> connections, int timeout) {
	// Nasty hack. Check every (timeout/10)ms for data.
	struct timespec wait;
	int step = timeout/10;
	wait.tv_sec = step/1000;
	wait.tv_nsec = step%1000 * 1000000;
	for (int i=0;i<10;++i) {
	  if (isDataAvailable(connections)) return true;
	  // Wait for a bit
	  nanosleep(&wait,NULL);
	}
	return false;
  }

  Address::Address(const char* host, int port) {
	m_port = htons(port);
	if (host) {
	  struct hostent* hostent = gethostbyname(host);
	  struct in_addr* address = (struct in_addr*)hostent->h_addr_list[0];
	  m_address = address->s_addr;
	}
	else {
	  m_address= INADDR_ANY;
	}
	makeName();
  }

  Address::Address(int host, int port) {
	m_port = port;
	m_address = host;
	makeName();
  }

  Address::Address(const Address& rhs) {
	m_port = rhs.m_port;
	m_address = rhs.m_address;
	makeName();
  }

  Address::Address(struct sockaddr_in address) {
	m_port = address.sin_port;
	m_address = address.sin_addr.s_addr;
	makeName();
  }

  Address::Address() {
	m_port = 0;
	m_address = 0;
	snprintf(name,256,"No address");
  }

  Address::~Address() {
  }

  void Address::makeName() {
	struct in_addr address;
	address.s_addr = m_address;
	snprintf(name,256,"%s:%d",inet_ntoa(address),ntohs(m_port));
  }

  bool Address::operator==(const Address& rhs) const {
	return m_port==rhs.m_port && m_address==rhs.m_address;
  }

  bool Address::operator< (const Address& rhs) const {
	if (m_address != rhs.m_address)
	  return m_address < rhs.m_address;
	return m_port < rhs.m_port;
  }

  void Address::fill(struct sockaddr_in* address) const {
	address->sin_family = AF_INET;
	address->sin_port = m_port;
	address->sin_addr.s_addr = m_address;
  }

  u_int16_t Address::port() const {
	return m_port;
  }

  u_int32_t Address::address() const {
	return m_address;
  }

  const char* Address::toString() const {
	return name;
  }
}
