#ifndef RESCUE_CONNECTION_H
#define RESCUE_CONNECTION_H

#include "common.h"
#include "input.h"
#include "output.h"
#include <string>
#include <netinet/in.h>
#include <vector>

namespace Librescue {
  class Address {
  private:
	u_int16_t m_port;
	u_int32_t m_address;
	char name[256];

	void makeName();

  public:
	Address(const char* hostname, int port);
	// The host and port are in network byte order.
	Address(int host, int port);
	Address(const Address& rhs);
	Address(struct sockaddr_in address);
	Address();
	virtual ~Address();

	u_int16_t port() const;
	u_int32_t address() const;

	void fill(struct sockaddr_in* address) const;
	const char* toString() const;

	//	Address& operator= (const Address& rhs);
	bool operator== (const Address& rhs) const;
	bool operator< (const Address& rhs) const;
  };

  class Connection {
  private:
	// No copying allowed
	Connection(const Connection& rhs);
	Connection& operator=(const Connection& rhs);

  protected:
	Connection();

  public:
	virtual ~Connection();

	virtual bool send(const Output& buffer) = 0;
	//	virtual bool send(std::string string);
	virtual void receive(Input& buffer) = 0;
	//	virtual std::string receive();
	virtual bool isDataAvailable(int timeout) = 0;

	virtual void close() = 0;
	virtual bool isOpen() = 0;

	// Get the Address that the last message was received from
	virtual const Address& addressReceivedFrom() const = 0;

	// Find out if any of a set of connections has data available now
	static bool isDataAvailable(std::vector<Connection*> connections);
	// Find out if any of a set of connections has data available. If no data is available, block until data is available or until "timeout" milliseconds have passed. A timeout of less than zero will never timeout.
	static bool isDataAvailable(std::vector<Connection*> connections, int timeout);
  };
}

#endif
