#ifndef RESCUE_TCP_H
#define RESCUE_TCP_H

#include <string>
#include <list>
#include <pthread.h>
#include "input.h"
#include "output.h"
#include "connection.h"

namespace Librescue
{
  /*
  typedef struct AcceptInfo {
	struct sockaddr_in from;
	int socket;
  } AcceptInfo;
  */

  class TcpConnection : public Connection
	{
	private:
	  // No copying allowed
	  TcpConnection(const TcpConnection& rhs); 
	  void operator=(const TcpConnection& rhs);

	  void init(const Address* target);
	  bool receiveData(Byte* buffer, int length);

	protected:
	  int m_socketDescriptor;
	  Address m_from;
	  Address m_local;

	public:
	  TcpConnection(const char* host, int port);
	  TcpConnection(const Address& target);
	  TcpConnection(const Address& from, int socketDescriptor);
	  virtual ~TcpConnection();

	  virtual bool send(const Output& buffer);
	  virtual void receive(Input& buffer);
	  virtual bool isDataAvailable(int timeout);
	  virtual void close();
	  virtual bool isOpen();

	  // Get the Address that the last message was received from
	  virtual const Address& addressReceivedFrom() const;
	};

  class TcpServer {
  public:
	class TCPConnectionCallback {
	public:
	  virtual void incomingTCPConnection(TcpConnection* connection, const Address& from) = 0;
	};

  private:
	int m_socket;
	pthread_t m_listenThread;
	//	pthread_mutex_t m_listenLock;
	pthread_mutex_t m_runLock;
	bool running;
	TCPConnectionCallback& m_callback;

	void loop();
	static void* threadFunc(void* server);

	// No copying allowed
	TcpServer(const TcpServer& rhs);
	void operator=(const TcpServer& rhs);

	//  protected:
	//	std::list<AcceptInfo> accepted;

  public:
	TcpServer(int listenPort,TCPConnectionCallback& callback);
	virtual ~TcpServer();

	void start();
	void stop();

	//	bool hasConnections();
	// Gets a newly allocated TcpConnection, or NULL if there are no connections waiting
	//	TcpConnection* getConnection();
  };
}

#endif

